function wb = wishart_blockdiag(X,L,varargin)

% Wishart based detection of reflection symmetry in polSAR covmat data
%
% wb = wishart_blockdiag(X,L);
%
% ShhShh* ShhShv* ShhSvv*     k    a   rho
% ShvShh* ShvShv* ShvSvv*  =  a*   ksi b
% SvvShh* SvvShv* SvvSvv*     rho* b*  zeta
%
% SAR input images are images of the (row-wise) upper-right elements of
% the covariance matrix, e.g. for full polarimetry:
% a vector with nine real (i.e., non-complex) elements, namely
% [ShhShh* Re(ShhShv*) Im(ShhShv*) Re(ShhSvv*) Im(ShhSvv*) ...
%   ShvShv* Re(ShvSvv*) Im(ShvSvv*) SvvSvv*]
% = [k Re(a) Im(a) Re(rho) Im(rho) ksi Re(b) Im(b) zeta]
%
% Input
%   X	- covariance matrix input
%   L	- equivalent number of looks (ENL) for X
%   Pt  - logical, output p-values (and rho, omega2 and f), default: true
%
% Output: struct wb with fields
%   lnQ     - logarithm of test quantity Q
%   L       - equivalent number of looks, ENL (from input)
%   rho     - aux variable
%   omega2  - aux variable
%   f       - number of degrees of freedom
%   P       - probability of finding smaller -2 \rho \ln Q
%
% Reference
%
% Paul Connetable, Knut Conradsen, Allan A. Nielsen and Henning Skriver (2022).
% Test statistics for reflection symmetry, applications to quad-polarimetric
% SAR data for detection of man-made structures.
% Accepted for
% IEEE Journal of Selected Topics in Applied Earth Observations and
% Remote Sensing.

% (c) Copyright 2021
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, people.compute.dtu.dk/alan
% 6 Mar 2021

if nargin<2
    help wishart_blockdiag
    error('wishart_blockdiag: wrong input');
end
Ptest = true;
if nargin>2
    Ptest = varargin{1};
end
if nargout>1
    help wishart_blockdiag
    error('wishart_blockdiag: wrong output');
end

if ischar(X), X = freadenvisar(X); end

pX = size(X);
nvars = pX(3);

if nvars==9 % quad/full pol data
%     fprintf('wishart_blockdiag: quad/full pol\n');
    wb.lnQ = L*(log(wishart_det(X))-log(wishart_det(X(:,:,[1 4 5 9])))-log(X(:,:,6)));
    p  = 3;
    p1 = 2;
    p2 = 1;
elseif nvars==4 % dual pol data
%     fprintf('wishart_blockdiag: dual pol\n');
    wb.lnQ = L*(log(wishart_det(X))-log(X(:,:,1))-log(X(:,:,4)));
    p  = 2;
    p1 = 1;
    p2 = 1;
else
    error('wishart_blockdiag: input must be 9 or 4 (real) bands');
end

clear X
wb.L = L;
if Ptest
    psump2 = p^2 - (p1^2 + p2^2);
    psump3 = p^3 - (p1^3 + p2^3);
    psump4 = p^4 - (p1^4 + p2^4);
    wb.rho = 1-psump3/psump2/3/L;
    wb.omega2 = (-psump3^2/psump2/36 + psump4/24)/L^2/wb.rho^2;
    wb.f = psump2;
    % wb.rho = 1;
    % wb.omega2 = 0;
    wb.P = (1 - wb.omega2)*gammainc(-wb.rho*wb.lnQ,0.5*wb.f  ) ...
              + wb.omega2 *gammainc(-wb.rho*wb.lnQ,0.5*wb.f+2);
end
